@@ -4,7 +4,7 @@ use std::rc::Rc;
 
 pub mod attributes;
 pub mod component;
-mod component_node;
+pub mod component_node;
 pub mod events;
 
 pub use component::Component;
@@ -22,16 +22,23 @@ pub enum Html<DemirootComp: Component> {
     ElementNode {
         tag_name: String,
         children: Vec<Self>,
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
-        ref_marker: Vec<RefMarker<DemirootComp>>,
     },
     Fragment(Vec<Self>),
 }
 
+/// Attributes for Html<Msg>
+#[derive(Clone)]
+pub struct Attributes<DemirootComp: Component> {
+    attributes: node::Attributes,
+    key: Option<String>,
+    ref_marker: Vec<RefMarker<DemirootComp>>,
+}
+
 pub enum RefMarker<DemirootComp: Component> {
     RefString(RefString<DemirootComp>),
-    WrappedRef(Box<dyn FnOnce(web_sys::Node)>),
+    WrappedRef(WrappedRef),
 }
 
 pub struct RefString<DemirootComp: Component> {
@@ -39,10 +46,8 @@ pub struct RefString<DemirootComp: Component> {
     __phantom_demiroot: std::marker::PhantomData<DemirootComp>,
 }
 
-/// Attributes for Html<Msg>
-#[derive(Clone)]
-pub struct Attributes {
-    attributes: node::Attributes,
+pub struct WrappedRef {
+    data: Rc<RefCell<Box<dyn FnMut(web_sys::Node)>>>,
 }
 
 /// Events for Html<Msg>
@@ -57,7 +62,7 @@ pub enum EventHandler<Msg> {
 
 pub enum ComponentNode<DemirootComp: Component> {
     PackedComponentNode(Box<dyn PackedComponentNode<DemirootComp = DemirootComp>>),
-    AssembledComponentNode(AssembledComponentNode<DemirootComp>),
+    PrepackedComponentNode(Box<dyn PrepackedComponentNode<DemirootComp = DemirootComp>>),
     WrappedAssembledComponentNode(WrappedAssembledComponentNode<DemirootComp>),
 }
 
@@ -66,7 +71,10 @@ pub trait PackedComponentNode {
     fn assemble(
         &mut self,
         before: Option<Rc<RefCell<dyn AssembledChildComponent<DemirootComp = Self::DemirootComp>>>>,
-    ) -> AssembledComponentNode<Self::DemirootComp>;
+    ) -> (
+        Rc<RefCell<dyn AssembledChildComponent<DemirootComp = Self::DemirootComp>>>,
+        Vec<Html<Self::DemirootComp>>,
+    );
 }
 
 pub struct PackedComponentNodeInstance<ThisComp: Component, DemirootComp: Component> {
@@ -80,12 +88,26 @@ struct PackedComponentNodeInstanceData<ThisComp: Component, DemirootComp: Compon
     children: Vec<Html<DemirootComp>>,
 }
 
-pub struct WrappedPackedComponentNode<SuperDemirootComp: Component> {
-    data: Box<dyn PackedComponentNode<DemirootComp = SuperDemirootComp>>,
+pub trait PrepackedComponentNode {
+    type DemirootComp: Component;
+
+    fn assemble(
+        &mut self,
+        before: Option<Rc<RefCell<dyn AssembledChildComponent<DemirootComp = Self::DemirootComp>>>>,
+    ) -> (
+        Rc<RefCell<dyn AssembledChildComponent<DemirootComp = Self::DemirootComp>>>,
+        Vec<Html<Self::DemirootComp>>,
+    );
 }
 
-pub struct AssembledComponentNode<DemirootComp: Component> {
-    data: Rc<RefCell<dyn AssembledChildComponent<DemirootComp = DemirootComp>>>,
+pub struct PrepackedComponentNodeInstance<ThisComp: Component, DemirootComp: Component> {
+    data: Option<PrepackedComponentNodeInstanceData<ThisComp, DemirootComp>>,
+}
+
+struct PrepackedComponentNodeInstanceData<ThisComp: Component, DemirootComp: Component> {
+    data: Rc<RefCell<ThisComp>>,
+    props: ThisComp::Props,
+    sub_mapper: component::Sub<ThisComp::Sub, DemirootComp::Msg>,
     children: Vec<Html<DemirootComp>>,
 }
 
@@ -94,6 +116,24 @@ pub struct WrappedAssembledComponentNode<VDemirootComp: Component> {
     rendered: std::collections::VecDeque<crate::kagura::Node>,
 }
 
+impl<DemirootComp: Component> Clone for RefMarker<DemirootComp> {
+    fn clone(&self) -> Self {
+        match self {
+            Self::RefString(data) => Self::RefString(data.clone()),
+            Self::WrappedRef(data) => Self::WrappedRef(data.clone()),
+        }
+    }
+}
+
+impl<DemirootComp: Component> Clone for RefString<DemirootComp> {
+    fn clone(&self) -> Self {
+        Self {
+            name: self.name.clone(),
+            __phantom_demiroot: std::marker::PhantomData,
+        }
+    }
+}
+
 impl<DemirootComp: Component> RefString<DemirootComp> {
     pub fn new(name: String) -> Self {
         Self {
@@ -103,6 +143,22 @@ impl<DemirootComp: Component> RefString<DemirootComp> {
     }
 }
 
+impl Clone for WrappedRef {
+    fn clone(&self) -> Self {
+        Self {
+            data: Rc::clone(&self.data),
+        }
+    }
+}
+
+impl WrappedRef {
+    pub fn new(f: impl FnMut(web_sys::Node) + 'static) -> Self {
+        Self {
+            data: Rc::new(RefCell::new(Box::new(f))),
+        }
+    }
+}
+
 impl<DemirootComp: Component> std::fmt::Debug for Html<DemirootComp> {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         match self {
@@ -141,7 +197,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     /// Creates Html from element
     pub fn node(
         tag_name: impl Into<String>,
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -150,7 +206,6 @@ impl<DemirootComp: Component> Html<DemirootComp> {
             children,
             attributes,
             events,
-            ref_marker: vec![],
         }
     }
 
@@ -159,15 +214,8 @@ impl<DemirootComp: Component> Html<DemirootComp> {
         Html::Fragment(vec![])
     }
 
-    pub fn ref_name(mut self, name: impl Into<String>) -> Self {
-        if let Self::ElementNode { ref_marker, .. } = &mut self {
-            ref_marker.push(RefMarker::RefString(RefString::new(name.into())));
-        }
-        self
-    }
-
     pub fn a(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -175,7 +223,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn abbr(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -183,7 +231,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn address(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -191,7 +239,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn area(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -199,7 +247,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn article(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -207,7 +255,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn aside(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -215,7 +263,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn audio(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -223,7 +271,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn b(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -231,7 +279,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn bdi(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -239,7 +287,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn bdo(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -247,7 +295,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn blockquote(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -255,7 +303,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn button(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -263,7 +311,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn br(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -271,7 +319,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn cite(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -279,7 +327,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn caption(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -287,7 +335,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn canvas(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -295,7 +343,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn code(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -303,7 +351,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn col(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -311,7 +359,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn colgroup(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -319,7 +367,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn datalist(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -327,7 +375,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn details(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -335,7 +383,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn dd(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -343,7 +391,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn dfn(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -351,7 +399,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn div(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -359,7 +407,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn data(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -367,7 +415,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn del(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -375,7 +423,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn dl(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -383,7 +431,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn dt(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -391,7 +439,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn em(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -399,7 +447,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn embed(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -407,7 +455,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn fieldset(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -415,7 +463,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn figcaption(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -423,7 +471,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn figure(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -431,7 +479,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn footer(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -439,7 +487,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn form(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -447,7 +495,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn h1(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -455,7 +503,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn h2(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -463,7 +511,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn h3(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -471,7 +519,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn h4(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -479,7 +527,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn h5(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -487,7 +535,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn h6(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -495,7 +543,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn header(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -503,7 +551,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn hr(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -511,7 +559,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn i(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -519,7 +567,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn iframe(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -527,7 +575,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn img(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -535,7 +583,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn input(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -543,7 +591,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn ins(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -551,7 +599,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn kbd(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -559,7 +607,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn label(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -567,7 +615,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn legend(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -575,7 +623,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn li(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -583,7 +631,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn main(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -591,7 +639,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn mark(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -599,7 +647,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn map(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -607,7 +655,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn menu(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -615,7 +663,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn menuitem(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -623,7 +671,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn meter(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -631,7 +679,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn nav(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -639,7 +687,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn object(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -647,7 +695,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn ol(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -655,7 +703,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn optgroup(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -663,7 +711,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn option(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -671,7 +719,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn output(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -679,7 +727,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn p(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -687,7 +735,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn param(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -695,7 +743,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn picture(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -703,7 +751,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn pre(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -711,7 +759,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn progress(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -719,7 +767,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn q(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -727,7 +775,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn rb(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -735,7 +783,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn rp(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -743,7 +791,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn rt(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -751,15 +799,15 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn rtc(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
         Self::node("rtc", attributes, events, children)
     }
 
-    pub fn rubu(
-        attributes: Attributes,
+    pub fn ruby(
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -767,7 +815,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn s(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -775,7 +823,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn samp(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -783,7 +831,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn section(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -791,7 +839,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn select(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -799,7 +847,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn small(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -807,7 +855,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn source(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -815,7 +863,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn span(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -823,7 +871,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn strong(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -831,7 +879,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn sub(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -839,7 +887,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn summary(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -847,7 +895,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn sup(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -855,7 +903,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn table(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -863,7 +911,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn tbody(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -871,7 +919,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn td(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -879,7 +927,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn textarea(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -887,7 +935,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn tfoot(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -895,7 +943,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn th(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -903,7 +951,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn thead(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -911,7 +959,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn time(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -919,7 +967,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn tr(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -927,7 +975,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn track(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -935,7 +983,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn u(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -943,7 +991,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn ul(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -951,7 +999,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn var(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -959,7 +1007,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn video(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
@@ -967,7 +1015,7 @@ impl<DemirootComp: Component> Html<DemirootComp> {
     }
 
     pub fn wbr(
-        attributes: Attributes,
+        attributes: Attributes<DemirootComp>,
         events: Events<DemirootComp::Msg>,
         children: Vec<Html<DemirootComp>>,
     ) -> Self {
